package com.brilliance.isc.mda.dao;

import com.brilliance.isc.bo.Swm;
import com.brilliance.isc.vo.manager.SwmVo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

@Mapper
public interface SwmMapper {

    List<Swm> selectByWhere(Swm record);

    /**
     * 通过ID查询单条数据
     *
     * @param swm 实例
     * @return 实例对象
     */
    List<Swm> queryById(Swm swm);

    /**
     * 查询指定行数据
     *
     * @param swm 查询条件
     * @return 对象列表
     */
    List<Swm> queryAllByLimit(Swm swm);

    List<Swm> selectAlldst(Swm swm);

    //通过inr查询单条数据
    Swm queryByInr(String inr);

    /**
     * 统计总行数
     *
     * @param swm 查询条件
     * @return 总行数
     */
    long count(Swm swm);

    /**
     * 新增数据
     *
     * @param swm 实例对象
     * @return 影响行数
     */
    int insert(Swm swm);

    /**
     * 批量新增数据（MyBatis原生foreach方法）
     *
     * @param entities List<Swm> 实例对象列表
     * @return 影响行数
     */
    int insertBatch(@Param("entities") List<Swm> entities);

    /**
     * 修改数据
     *
     * @param swm 实例对象
     * @return 影响行数
     */
    int update(Swm swm);

    /**
     * 通过主键删除数据
     *
     * @param swmVo 主键
     * @return 影响行数
     */
    int deleteById(SwmVo swmVo);
    //批量删除
    int deleteAll(@Param("list") List list);

    int check(Swm swm);

    /**
     * 查询是否存在修改的记录
     *
     * @param mt 报文类型
     * @param fmt 报文格式
     * @param frm 交易码
     * @return 交易码
     */
    String selectFrmByWhere(@Param("fmt") String fmt,@Param("mt") String mt,@Param("frm") String frm);

    /**
     * 查报文类型配置对应的所有交易码
     *
     * @param fmt  报文格式
     * @param mt  报文类型
     * @param altmapflg  修改标志
     * @return 交易码
     */
    List<String> selectFrmListByWhere(@Param("fmt") String fmt,@Param("mt") String mt,@Param("altmapflg") String altmapflg);

    /**
     * 查报配置信息
     *
     * @param fmt  报文格式
     * @param mt  报文类型
     * @param frm  交易码
     * @param altmapflg  修改标志
     * @return 配置记录
     */
    List<Swm> selectSwmListByWhere(@Param("fmt") String fmt,@Param("mt") String mt,@Param("frm") String frm,@Param("altmapflg") String altmapflg);

}