package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.lg.GftcanStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.GftcanStoreVo;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Gftcan Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface GftcanStructMapper extends IBaseStructMapper<GftcanStoreBo, GftcanStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    GftcanStructMapper INSTANCE = Mappers.getMapper(GftcanStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    GftcanStoreBo voTransferToBo(GftcanStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mapping(target = "gidgrp", ignore = true)
    @Mapping(target = "gcdgrp", ignore = true)
    void voUpdateToBo(GftcanStoreVo vo, @MappingTarget GftcanStoreBo bo);

    @AfterMapping
    default void afterUpdate(GftcanStoreVo vo, @MappingTarget GftcanStoreBo bo) {
        bo.getGcdgrp().getRec().setClsdat(vo.getGcdgrp().getRec().getClsdat());
        bo.setRebkpflg(vo.getRebkpflg());
    }

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    GftcanStoreVo boTransferToVo(GftcanStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(GftcanStoreBo bo, @MappingTarget GftcanStoreVo vo);
}
