package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.bo.lg.HitdlaStoreBo;
import com.brilliance.isc.vo.lg.HitdlaStoreVo;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingTarget;
import org.mapstruct.Mappings;
import org.mapstruct.factory.Mappers;


/**
 * Gitame Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface HitdlaStructMapper extends IBaseStructMapper<HitdlaStoreBo, HitdlaStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    HitdlaStructMapper INSTANCE = Mappers.getMapper(HitdlaStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    HitdlaStoreBo voTransferToBo(HitdlaStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
            @Mapping(target = "oldnidgrp",ignore = true)
    })
    void voUpdateToBo(HitdlaStoreVo vo, @MappingTarget HitdlaStoreBo bo);

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    HitdlaStoreVo boTransferToVo(HitdlaStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(HitdlaStoreBo bo, @MappingTarget HitdlaStoreVo vo);
}
